<?php

/* 
Copyright 2009  Randy Hunt  (email : bbqiguana@gmail.com)

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

Plugin Name: TweetPost
Description: Tweets a link to your post when a new post is made on your blog
Version: 1.0
Author: Randy Hunt
*/

require_once('twitter/OAuth.php');
require_once('twitter/twitteroauth.php');

if ( is_admin() ) { // admin actions
	define('TWEETPOST_PATH', get_option('siteurl') . '/wp-content/plugins/' . dirname(plugin_basename(__FILE__)));
	define('TWEETPOST_PAGE', get_option('siteurl') . '/wp-admin/options-general.php?page=tweetpost');
	
	if ($_GET['page'] == 'tweetpost' && $_GET['oauth']=='authenticate') {
		tweetpost_authenticate_twitter();
	}
	if ($_REQUEST['page'] == 'tweetpost' && $_REQUEST['action'] == 'clearuser') {
		tweetpost_clear_user();
	}

	add_action('admin_menu', 'tweetpost_menu');
	add_action('admin_init', 'tweetpost_init');
	
	wp_enqueue_script("jquery");
	wp_enqueue_script("tweetpost", TWEETPOST_PATH . '/tweetpost.js', array('jquery'), '1.0');
}

function tweetpost_savepost ($post_id) {

    if (is_int($post_id)) $post = get_post($post_id);
    else $post = $post_id;

	//ignore revisions
	if ($post->post_type == 'revision') return;

	//ignore non-published data and password protected data
	if ($post->post_status != 'publish' || $post->post_password != '' ) return;

	$title = $post->post_title;
	$link = get_permalink($post_id);

	$shortener = get_option('tweetpost_url_shortener');
	if($shortener == 'bitly') {
		$shorturl = tweetpost_bitly($link);
	} else 
	if($shortener == 'supr') {
		$shorturl = tweetpost_supr($link);
	} else
		$shorturl = $link;
	if (!$shorturl) return;

	//get settings
	$twitter_username = get_option('tweetpost_twitter_user');
	$twitter_password = get_option('tweetpost_twitter_pass');
	$twitterer = get_usermeta($post->post_author, 'twitter');
	if ($twitterer) {
		$twitterer = '@'.$twitterer;
	} else {
		$user=get_userdata($post->post_author);
		$twitterer = $user->user_nicename;
	}

	//create Tweet...
	$consumer_key    = get_option('tweetpost_twitter_key'   );
	$consumer_secret = get_option('tweetpost_twitter_secret');
	$access_token    = get_option('tweetpost_access_token'   );
	$access_secret   = get_option('tweetpost_access_secret'  );

	$tweet = str_replace('@username', $twitterer, get_option('tweetpost_opt_introtxt')).' ';
	$limit = 140 - (strlen($tweet) + strlen($shorturl) + 3);
	if (strlen($title) > $limit) {
		$title = substr($title, 0, $limit - 3) . '...';
	}
	$tweet = $tweet . $title . ' - ' . $shorturl;
	$connection = new TwitterOAuth($consumer_key, $consumer_secret, $access_token, $access_secret);
	$parameters = array('status' => $tweet);
	$status = $connection->post('statuses/update', $parameters);
}

function tweetpost_bitly ($url) {
	$user = get_option('tweetpost_bitly_user');
	$apik = get_option('tweetpost_bitly_apikey');
	
	$endpoint = 'http://api.bit.ly/shorten?version=2.0.1&history=1&format=xml&login='.$user.'&apiKey='.$apik.'&longUrl='.$url;
	$xml = file_get_contents($endpoint);

	preg_match('/<errorcode>([^<]+)<\/errorcode>/i', $xml, $matches);
	if($matches && $matches[1]!='0') return false;
	
	preg_match('/<shorturl>([^<]+)<\/shorturl>/i', $xml, $matches);
	
	if($matches && $matches[1]) return $matches[1];
	else return false;
}

function tweetpost_supr ($url) {
	$user = get_option('tweetpost_supr_user');
	$apik = get_option('tweetpost_supr_apikey');

	$endpoint = 'http://su.pr/api/simpleshorten?url='.$url.'&login='.$user.'&apiKey='.$apik;
	$xml = file_get_contents($endpoint);
	if($xml && $xml != 'ERROR') return $xml;
	else return false;
}

function tweetpost_showuser ($user) {
	?>
	<h3>Extra profile information</h3>
	<table class="form-table">
	<tr><th><label for="twitter">Twitter</label></th><td>
	<input type="text" name="twitter" id="twitter" value="<?php echo esc_attr( get_the_author_meta('twitter', $user->ID ) ) ?>" class="regular-text" /><br />
	<span class="description">Please enter your Twitter username.</span>
	</td></tr></table>
	<?php
}

function tweetpost_edituser ($user_id) {
	if ( !current_user_can('edit_user', $user_id) )
		return false;

	/* Copy and paste this line for additional fields. Make sure to change 'twitter' to the field ID. */
	update_usermeta($user_id, 'twitter', $_POST['twitter'] );
}

function tweetpost_updtwitter ($userid, $twitterid) {
	update_usermeta ($userid, 'twitter', $twitterid);
}

function tweetpost_init () {
	register_setting('tweetpost', 'twitter_username');
	register_setting('tweetpost', 'twitter_password');
	register_setting('tweetpost', 'twitter_key');
	register_setting('tweetpost', 'twitter_secret');
	register_setting('tweetpost', 'bitly_username');
	register_setting('tweetpost', 'bitly_password');
	register_setting('tweetpost', 'bitly_apikey');
	register_setting('tweetpost', 'supr_username');
	register_setting('tweetpost', 'supr_password');
	register_setting('tweetpost', 'url_shortener');
	register_setting('tweetpost', 'opt_introtxt');
}

function tweetpost_install () {
  //$tweetpost_twitter_user  = get_option('tweetpost_twitter_user');
  //$tweetpost_twitter_pass  = get_option('tweetpost_twitter_pass');
  //$tweetpost_bitly_user    = get_option('tweetpost_bitly_user');
  //$tweetpost_bitly_pass    = get_option('tweetpost_bitly_pass');
  //$tweetpost_bitly_apikey  = get_option('tweetpost_bitly_apikey');
  //$tweetpost_supr_user     = get_option('tweetpost_supr_user');
  //$tweetpost_supr_apikey   = get_option('tweetpost_supr_apikey');
  $tweetpost_url_shortener = get_option('tweetpost_url_shortener');
  $tweetpost_opt_introtxt  = get_option('tweetpost_opt_introtxt');

  //if (!$tweetpost_twitter_user ) update_option('tweetpost_twitter_user', 'username');
  //if (!$tweetpost_twitter_pass ) update_option('tweetpost_twitter_pass', 'password');
  //if (!$tweetpost_bitly_user   ) update_option('tweetpost_bitly_user',   'username');
  //if (!$tweetpost_bitly_pass   ) update_option('tweetpost_bitly_pass',   'password');
  //if (!$tweetpost_bitly_apikey ) update_option('tweetpost_bitly_apikey', 'apikey'  );
  //if (!$tweetpost_supr_user    ) update_option('tweetpost_supr_user',    'username');
  //if (!$tweetpost_supr_apikey  ) update_option('tweetpost_supr_apikey',  'apikey'  );
  if (!$tweetpost_url_shortener) update_option('tweetpost_url_shortener', 'bitly'  );
  if (!$tweetpost_opt_introtxt ) update_option('tweetpost_opt_introtxt', 'New post from @username : ');
}

function tweetpost_adminnotice () {
	$twitter_key    = get_option('tweetpost_twitter_key');
 	$twitter_secret = get_option('tweetpost_twitter_secret');
	if ($_REQUEST['page'] != 'tweetpost' && (empty($twitter_key) || empty($twitter_secret))) {
		$twitter_user = get_option('tweetpost_twitter_user');
		if(!empty($twitter_user)) {
			echo '<div class="error"><p>The settings for Tweetpost have changed. <a href="'.TWEETPOST_PAGE.'">Click here</a> to configure the plugin.</p></div>';
		} else {
			echo '<div class="updated"><p>Tweetpost is not configured. <a href="'.TWEETPOST_PAGE.'">Click here</a> to configure the plugin.</p></div>';
		}
	}
}

function tweetpost_menu () {
	if ( function_exists('add_options_page') ) {
		add_options_page('TweetPost', 'TweetPost', 8, 'tweetpost', 'tweetpost_options');
	}
}

function tweetpost_clear_user () {
	update_option('tweetpost_access_token',  '');
	update_option('tweetpost_access_secret', '');
	update_option('tweetpost_twitter_uid',   '');
	update_option('tweetpost_screen_name',   '');
}

function tweetpost_accept_twitter () {
	$twitter_key    = get_option('tweetpost_twitter_key');
	$twitter_secret = get_option('tweetpost_twitter_secret');
	$request_token  = get_option('tweetpost_request_token');
	$request_secret = get_option('tweetpost_request_secret');
	$connection = new TwitterOAuth($twitter_key, $twitter_secret, $request_token, $request_secret);
	$access_token = $connection->getAccessToken($_REQUEST['oauth_verifier']);
	update_option('tweetpost_request_token', '');
	update_option('tweetpost_request_secret','');
	update_option('tweetpost_access_token',  $access_token['oauth_token']       );
	update_option('tweetpost_access_secret', $access_token['oauth_token_secret']);
	update_option('tweetpost_twitter_uid',   $access_token['user_id']           );
	update_option('tweetpost_screen_name',   $access_token['screen_name']       );
}

function tweetpost_authenticate_twitter () {
	$consumerkey = get_option('tweetpost_twitter_key');
	$consumersecret = get_option('tweetpost_twitter_secret');
	$connection = new TwitterOAuth($consumerkey, $consumersecret);
	$request_token = $connection->getRequestToken(TWEETPOST_PAGE);
	if($connection->http_code = '200' && is_array($request_token)) {
		$token = $request_token['oauth_token'];
		update_option('tweetpost_request_token', $token);
		update_option('tweetpost_request_secret', $request_token['oauth_token_secret']);
		$url = $connection->getAuthorizeUrl($token);
		header('Location: ' . $url);
	} else {
		echo '<strong>error connecting to Twitter.</strong>';
	}
}

function tweetpost_options () {
	if (!empty($_REQUEST['oauth_token']) && !empty($_REQUEST['oauth_verifier']))
		tweetpost_accept_twitter();

	echo "<div class=\"wrap\">";
	echo "<h2>Tweetpost</h2>";
	if ( ($_POST['action']=='update') ) {
		//check_admin_referer('externimg_update-action');
		update_option('tweetpost_twitter_user',  $_POST['twitter_username']);
		update_option('tweetpost_twitter_pass',  $_POST['twitter_password']);
		update_option('tweetpost_twitter_key',   $_POST['twitter_key']     );
		update_option('tweetpost_twitter_secret',$_POST['twitter_secret']  );
		update_option('tweetpost_bitly_user',    $_POST['bitly_username']  );
		update_option('tweetpost_bitly_pass',    $_POST['bitly_password']  );
		update_option('tweetpost_bitly_apikey',  $_POST['bitly_apikey']    );
		update_option('tweetpost_supr_user',     $_POST['supr_username']   );
		update_option('tweetpost_supr_apikey',   $_POST['supr_apikey']     );
		update_option('tweetpost_url_shortener', $_POST['url_shortener']   );
		update_option('tweetpost_opt_introtxt',  $_POST['opt_introtxt']    );
		echo '<div id="message" class="updated fade" style="background-color:rgb(255,251,204);"><p>Settings updated.</p></div>';
	}
	$consumerkey    = get_option('tweetpost_twitter_key'   );
	$consumersecret = get_option('tweetpost_twitter_secret');
	$twitter_uid    = get_option('tweetpost_twitter_uid'   );
	$screen_name    = get_option('tweetpost_screen_name'   );

	echo '<form name="tweetpost-options" method="post" action="">';
	settings_fields('tweetpost');
	$tweetpost_shortener = get_option('tweetpost_url_shortener');
	if($tweetpost_shortener == '') $tweetpost_shortener = 'bitly';

	?>
	<big>Twitter</big>
	<table class="form-table">
	<tr><th><label for="twitter_key">Twitter Consumer Key</label></th>
	<td><input type="text" size="15" name="twitter_key" id="twitter_key" value="<?php echo esc_attr($consumerkey) ?>" class="regular-text" /><br/>
		<span id="twitterkeymsg" class="description"<?php if ($consumerkey && $consumersecret) {echo ' style="display:none;"';} ?>>Don't have keys? Get 'em <a href="http://dev.twitter.com/apps/new">here</a>.</span></td></tr>
	<tr><th><label for="twitter_secret">Twitter Consumer Secret</label></th>
	<td><input type="text" size="15" name="twitter_secret" id="twitter_secret" value="<?php echo esc_attr($consumersecret) ?>" class="regular-text" /><br/>
		<span class="description"></span></td></tr>
	<tr><th><label>Global Twitter Account</label></th>
	<td><span<?php if($screen_name) { echo ' style="display:none"'; } ?>>
	<a href="<?php echo TWEETPOST_PAGE; ?>&oauth=authenticate" title="Authorize"><img src="<?php echo TWEETPOST_PATH.'/authorize.png'; ?>" alt="authorize"></a></span>
	<?php if ($screen_name) { echo "<strong>@$screen_name</strong> <a href=\"" .TWEETPOST_PAGE."&action=clearuser\">change</a>"; } ?></td></tr>
	<tr><td>&nbsp;</td><td>&nbsp;</td></tr>
	</table>

	<big>URL shortening</big>
	<table class="form-table">
	<tr><th><label for="url_shortener">URL Shortener</label></th>
	<td><select name="url_shortener" id="url_shortener">
		<option value="bitly" <?php if($tweetpost_shortener=='bitly') echo 'selected'; ?>>bit.ly</option>
		<option value="supr" <?php if($tweetpost_shortener=='supr') echo 'selected'; ?>>su.pr</option>
	</select>
	</td></tr>

<tbody id="tweetpost_bitly_settings" <?php if($tweetpost_shortener!='bitly') echo 'style="display:none"'; ?>>
	<tr><th><label for="bitly_username">Bit.ly Username</label></th>
	<td><input type="text" size="15" name="bitly_username" id="bitly_username" value="<?php echo esc_attr(get_option('tweetpost_bitly_user')) ?>" class="regular-text" /><br />
		<span class="description">Please enter the Bit.ly username for your site.</span>
	</td></tr>
	<tr><th><label for="bitly_apikey">Bit.ly API key</label></th>
	<td><input type="text" size="20" name="bitly_apikey" id="bitly_apikey" value="<?php echo esc_attr(get_option('tweetpost_bitly_apikey')) ?>" class="regular-text" /><br />
		<span class="description">Please enter the Bit.ly API key for your site.</span>
	</td></tr>
	<tr><td>&nbsp;</td><td><span id="tweetpost_bitlyresult" style="padding:2px"></span></td></tr>
</tbody>
<tbody id="tweetpost_supr_settings" <?php if($tweetpost_shortener!='supr') echo 'style="display:none"'; ?>>
	<tr><th><label for="supr_username">Su.pr Username</label></th>
	<td><input type-"text" size="15" name="supr_username" id="supr_username" value="<?php echo esc_attr(get_option('tweetpost_supr_user')) ?>" class="regular-text" /><br />
		<span class="description">Please enter the Su.pr username for your site.</span>
	</td></tr>
	<tr><th><label for="supr_apikey">Su.pr API key</label></th>
	<td><input type="text" size="20" name="supr_apikey" id="supr_apikey" value="<?php echo esc_attr(get_option('tweetpost_supr_apikey')) ?>" class="regular-text" /><br />
		<span class="description">Please enter the Su.pr API key for your site.</span>
	<tr><td>&nbsp;</td><td><span id="tweetpost_suprresult" style="padding:2px"></span></td></tr>
</tbody>
	</table>

    <big>Options</big>
    <table class="form-table">
    <tr><th><label for="opt_introtxt">Intro text</label></th>
    <td><input type="text" size="15" name="opt_introtxt" id="opt_introtxt" value="<?php echo esc_attr(get_option('tweetpost_opt_introtxt')); ?>" class="regular-text" /><br/>
        <span class="description">Text to introduce the post. <em>eg: "New post from @username"</em><br/>
        <em>@username</em> will be replaced by the user's Twitter name, as specified on their profile</span></td></tr>
    </table>

	<div class="submit">
		<input type="hidden" name="externimg_update" value="action" />
		<input type="submit" name="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
	</div>
	</form>
	<div class="wrap">
		<big>Donate</big>
		<p>If you like this plugin consider donating a small amount to the author using PayPal to support further plugin development.</p>
		<div align="center"><form name="_xclick" action="https://www.paypal.com/us/cgi-bin/webscr" method="post"><input type="hidden" name="cmd" value="_xclick"><input type="hidden" name="business" value="bbqiguana@gmail.com"><input type="hidden" name="item_name" value="Donations for TweetPost plugin"><input type="hidden" name="currency_code" value="USD"><input type="image" src="http://www.paypal.com/en_US/i/btn/btn_donate_LG.gif" border="0" name="submit" alt="Make payments with PayPal - it's fast, free and secure!"></form></div>
		<p>If you think donating money is somehow impersonal you could also choose items from my <a href="http://www.amazon.com/registry/wishlist/18LMHOMRM49P8/ref=cm_wl_act_vv">Amazon.com wishlist</a>.</p>
	</div>

	</div>
	<?php
}

function tweetpost_admin_update () {
	//
}

register_activation_hook(__FILE__, 'tweetpost_install');

add_action('show_user_profile',        'tweetpost_showuser');
add_action('edit_user_profile',        'tweetpost_showuser');
add_action('personal_options_update',  'tweetpost_edituser');
add_action('edit_user_profile_update', 'tweetpost_edituser');
add_action('admin_notices',      'tweetpost_adminnotice'   );
add_action('new_to_publish',     'tweetpost_savepost', 10, 1);
add_action('draft_to_publish',   'tweetpost_savepost', 10, 1);
add_action('pending_to_publish', 'tweetpost_savepost', 10, 1);
add_action('future_to_publish',  'tweetpost_savepost', 10, 1);

?>
